"""Pull a YellowAnt application's data into a file"""
import json, yaml
import click

from yacli.cli import pass_context
from yacli.constants import APP_FILENAME, DEVELOPER_APPLICATION_ENDPOINT, DEVELOPER_APPLICATION_PULL_ENDPOINT
from yacli.common import save_app_config, save_application_yaml, AppIdInvalid, AppDataCorrupt,\
FilePermissionError
from yacli.helpers.save_data_from_request import save_data_from_request




@click.command()
@click.argument('app')
@click.option("--filename", default=APP_FILENAME, type=click.Path())
@pass_context
def cli(ctx, app, filename):
    """Pull application data from host in JSON format. Either provide application id or invoke name as argument."""
    try:
        ctx.is_auth_valid()

        try:
            # check if integer ID of application is passed
            app = int(app)
            req = ctx.get(DEVELOPER_APPLICATION_ENDPOINT + str(app) + "/")
        except:
            # check if app invoke name is passed
            app = app
            print("invoke name")
            req = ctx.get(DEVELOPER_APPLICATION_PULL_ENDPOINT + "?app=" + app)

        save_data_from_request(req, ctx)
    except Exception as e:
        ctx.log(e)



import click
import json, yaml

from yacli.cli import pass_context
from yacli.constants import APP_FILENAME, DEVELOPER_APPLICATION_SYNC_ENDPOINT
from yacli.common import save_app_config, save_application_yaml, AppDataCorrupt, FilePermissionError
from yacli.helpers.save_data_from_request import save_data_from_request


@click.command()
@click.option("--invoke_name", type=click.STRING, help="Unique name to invoke the app.")
@click.option("--website", type=click.STRING, help="Website of the app.")
@click.option("--api_url", type=click.STRING, help="API endpoint through which YellowAnt interacts with this app.")
@click.option("--install_page_url", type=click.STRING, help="Web URL through which users can integrate this app.")
@click.option("--privacy_policy_url", type=click.STRING, help="Web URL for the privacy policy of this app.")
@click.option("--redirect_uris", type=click.STRING, help="Redirect URI at which YellowAnt will provide OAuth tokens.")
@click.option("--quiet", "-q", is_flag=True, default=False, help="Do not prompt for inputs.")
@click.option("--filename", default=APP_FILENAME, type=click.File("rb"), help="App YAML file name.")
@pass_context
def cli(ctx, invoke_name, website, api_url, install_page_url, privacy_policy_url, redirect_uris, quiet, filename):
    """Create or update an application on YellowAnt"""
    # check permissions
    ctx.is_auth_valid()

    try:
        # load data from YAML file
        data = yaml.safe_load(filename.read())

        # check if no prompt is required from the user
        if quiet is True:
            data["invoke_name"] = invoke_name or data.get("invoke_name")
            data["website"] = website or data.get("website")
            data["api_url"] = api_url or data.get("api_url")
            data["install_page_url"] = install_page_url or data.get("install_page_url")
            data["privacy_policy_url"] = privacy_policy_url or data.get("privacy_policy_url")
            data["redirect_uris"] = redirect_uris or data.get("redirect_uris")
        else:
            data["invoke_name"] = invoke_name or click.prompt("Invoke Name", default=data.get("invoke_name"))
            data["website"] = website or click.prompt("Website", default=data.get("website"))
            data["api_url"] = api_url or click.prompt("API URL", default=data.get("api_url"))
            data["install_page_url"] = install_page_url or click.prompt("Installation URL", default=data.get("install_page_url"))
            data["privacy_policy_url"] = privacy_policy_url or click.prompt("Privacy Policy URL", default=data.get("privacy_policy_url"))
            data["redirect_uris"] = redirect_uris or click.prompt("Redirect URL", default=data.get("redirect_uris"))
        
        # send request to API
        req = ctx.post(DEVELOPER_APPLICATION_SYNC_ENDPOINT, data=json.dumps(data))

        save_data_from_request(req, ctx)
    except Exception as e:
        ctx.log(e)



import json
from yacli.common import AppDataCorrupt, FilePermissionError, save_application_yaml


def save_data_from_request(request, ctx):
    if str(request.status_code).startswith("2"):
        # application was created or updated
        try:
            try: # python 2.7 or >=3.6
                application_json = json.loads(request.content)
            except: # python 3.4 and 3.5
                application_json = json.loads(str(request.content, "utf-8"))
        except:
            raise AppDataCorrupt("Application data could not be parsed.")

        # log client ID and secret to console and save them in a json
        try:
            application_id = application_json.pop("id")
            client_id = application_json.pop("client_id")
            client_secret = application_json.pop("client_secret")
            verification_token = application_json.pop("verification_token")
            rtm_token = application_json.pop("rtm_token")
        except:
            raise AppDataCorrupt("Application data is incomplete.")
        # log values
        ctx.log("APP ID: " + str(application_id))
        ctx.log("CLIENT ID: " + client_id)
        ctx.log("CLIENT SECRET: " + client_secret)
        ctx.log("VERIFICATION TOKEN: " + verification_token)
        ctx.log("RTM TOKEN: " + rtm_token)
        # save credentials json
        credentials = {
            "application_id": application_id,
            "client_id": client_id,
            "invoke_name": application_json["invoke_name"],
            "client_secret": client_secret,
            "verification_token": verification_token,
            "rtm_token": rtm_token
        }
        try:
            with open("yellowant_app_credentials.json", "w") as credentials_json:
                json.dump(credentials, credentials_json, indent=4, sort_keys=True)
        except:
            raise FilePermissionError("Could not save credentials information properly.")
        # save app YAML and config details
        try:
            save_application_yaml(application_json)
            # save_app_config(application_id, application_json["invoke_name"])
        except Exception as e:
            raise FilePermissionError("Could not save application data a local file. Please check permissions.")
        # log final message to user
        action = "created" if request.status_code == 201 else "updated"
        ctx.log("Successfully {} application {} with your YellowAnt developer account."
                .format(action, application_json["invoke_name"]))
    elif request.status_code == 400:
        # there was an error in the application data
        try:
            # try to parse JSON error data
            error = json.loads(request.text)
            msg = "\nError: Please check your app data JSON.\n"
            for key_name in error:
                msg += key_name + ": " + str(error[key_name][0]) + "\n"
            raise Exception(msg)
        except:
            # error is just in plain text
            raise Exception(request.text)
    else:
        # some other error
        try:
            error = json.loads(request.text)
            raise Exception(error["detail"])
        except:
            error = request.text
            raise Exception(error)


from twython import Twython, TwythonError, TwythonAuthError

from .config import app_key, app_secret, screen_name, unittest


class TwythonAuthTestCase(unittest.TestCase):
    def setUp(self):
        self.api = Twython(app_key, app_secret)
        self.bad_api = Twython('BAD_APP_KEY', 'BAD_APP_SECRET')
        self.bad_api_invalid_tokens = Twython('BAD_APP_KEY', 'BAD_APP_SECRET',
                                              'BAD_OT', 'BAD_OTS')

        self.oauth2_api = Twython(app_key, app_secret, oauth_version=2)
        self.oauth2_bad_api = Twython('BAD_APP_KEY', 'BAD_APP_SECRET',
                                      oauth_version=2)

    @unittest.skip('skipping non-updated test')
    def test_get_authentication_tokens(self):
        """Test getting authentication tokens works"""
        self.api.get_authentication_tokens(callback_url='http://google.com/',
                                           force_login=True,
                                           screen_name=screen_name)

    @unittest.skip('skipping non-updated test')
    def test_get_authentication_tokens_bad_tokens(self):
        """Test getting authentication tokens with bad tokens
        raises TwythonAuthError"""
        self.assertRaises(TwythonAuthError, self.bad_api.get_authentication_tokens,
                          callback_url='http://google.com/')

    @unittest.skip('skipping non-updated test')
    def test_get_authorized_tokens_bad_tokens(self):
        """Test getting final tokens fails with wrong tokens"""
        self.assertRaises(TwythonError, self.bad_api.get_authorized_tokens,
                          'BAD_OAUTH_VERIFIER')

    @unittest.skip('skipping non-updated test')
    def test_get_authorized_tokens_invalid_or_expired_tokens(self):
        """Test getting final token fails when invalid or expired tokens have been passed"""
        self.assertRaises(TwythonError, self.bad_api_invalid_tokens.get_authorized_tokens,
                         'BAD_OAUTH_VERIFIER')

    @unittest.skip('skipping non-updated test')
    def test_get_authentication_tokens_raises_error_when_oauth2(self):
        """Test when API is set for OAuth 2, get_authentication_tokens raises
        a TwythonError"""
        self.assertRaises(TwythonError, self.oauth2_api.get_authentication_tokens)

    @unittest.skip('skipping non-updated test')
    def test_get_authorization_tokens_raises_error_when_oauth2(self):
        """Test when API is set for OAuth 2, get_authorized_tokens raises
        a TwythonError"""
        self.assertRaises(TwythonError, self.oauth2_api.get_authorized_tokens,
                          'BAD_OAUTH_VERIFIER')

    @unittest.skip('skipping non-updated test')
    def test_obtain_access_token(self):
        """Test obtaining an Application Only OAuth 2 access token succeeds"""
        self.oauth2_api.obtain_access_token()

    @unittest.skip('skipping non-updated test')
    def test_obtain_access_token_bad_tokens(self):
        """Test obtaining an Application Only OAuth 2 access token using bad app tokens fails"""
        self.assertRaises(TwythonAuthError,
                          self.oauth2_bad_api.obtain_access_token)

    @unittest.skip('skipping non-updated test')
    def test_obtain_access_token_raises_error_when_oauth1(self):
        """Test when API is set for OAuth 1, obtain_access_token raises a
        TwythonError"""
        self.assertRaises(TwythonError, self.api.obtain_access_token)



"""Save the authentication detaisl to the config file"""
import click
from yacli.cli import pass_context

try: # python 2 imports
    import ConfigParser
except: # python 3 imports
    import configparser as ConfigParser


def save_token(token, host, config_file):
    """Write token and host configs to a file"""
    config = ConfigParser.RawConfigParser()
    config.read(config_file)
    if not config.has_section("YELLOWANT_DEV"):
        config.add_section('YELLOWANT_DEV')
    config.set('YELLOWANT_DEV', 'token', token)
    config.set("YELLOWANT_DEV", "host", host)
    with open(config_file, 'w') as configfile:
        config.write(configfile)

    return token

@click.command()
@click.option('--token',
              hide_input=True,
              prompt='YellowAnt developer token',
              help="YellowAnt developer token.")
@click.option('--host',
              default="https://www.yellowant.com",
              prompt="YellowAnt Host URL",
              help="YellowAnt server host url.")
@pass_context
def cli(ctx, token, host):
    """Permanently save YellowAnt authentication details to a config file."""
    save_token(token, host, ctx.config_file)



# -*- coding: utf-8 -*-
from twython import Twython, TwythonError, TwythonAuthError, TwythonRateLimitError

from .config import (
    test_tweet_object, test_tweet_html, test_tweet_symbols_object,
    test_tweet_compat_object, test_tweet_extended_object, test_tweet_extended_html,
    unittest
)

import responses
import requests

from twython.compat import is_py2
if is_py2:
    from StringIO import StringIO
else:
    from io import StringIO

try:
    import unittest.mock as mock
except ImportError:
    import mock


class TwythonAPITestCase(unittest.TestCase):
    def setUp(self):
        self.api = Twython('', '', '', '')

    def get_url(self, endpoint):
        """Convenience function for mapping from endpoint to URL"""
        return '%s/%s.json' % (self.api.api_url % self.api.api_version, endpoint)

    def register_response(self, method, url, body='{}', match_querystring=False,
                          status=200, adding_headers=None, stream=False,
                          content_type='application/json; charset=utf-8'):
        """Wrapper function for responses for simpler unit tests"""

        # responses uses BytesIO to hold the body so it needs to be in bytes
        if not is_py2:
            body = bytes(body, 'UTF-8')

        responses.add(method, url, body, match_querystring,
                      status, adding_headers, stream, content_type)

    @responses.activate
    def test_request_should_handle_full_endpoint(self):
        """Test that request() accepts a full URL for the endpoint argument"""
        url = 'https://api.twitter.com/1.1/search/tweets.json'
        self.register_response(responses.GET, url)

        self.api.request(url)

        self.assertEqual(1, len(responses.calls))
        self.assertEqual(url, responses.calls[0].request.url)

    @responses.activate
    def test_request_should_handle_relative_endpoint(self):
        """Test that request() accepts a twitter endpoint name for the endpoint argument"""
        url = 'https://api.twitter.com/1.1/search/tweets.json'
        self.register_response(responses.GET, url)

        self.api.request('search/tweets', version='1.1')

        self.assertEqual(1, len(responses.calls))
        self.assertEqual(url, responses.calls[0].request.url)

    @responses.activate
    def test_request_should_post_request_regardless_of_case(self):
        """Test that request() accepts the HTTP method name regardless of case"""
        url = 'https://api.twitter.com/1.1/statuses/update.json'
        self.register_response(responses.POST, url)

        self.api.request(url, method='POST')
        self.api.request(url, method='post')

        self.assertEqual(2, len(responses.calls))
        self.assertEqual('POST', responses.calls[0].request.method)
        self.assertEqual('POST', responses.calls[1].request.method)

    @responses.activate
    def test_request_should_throw_exception_with_invalid_http_method(self):
        """Test that request() throws an exception when an invalid HTTP method is passed"""
        # TODO(cash): should Twython catch the AttributeError and throw a TwythonError
        self.assertRaises(AttributeError, self.api.request, endpoint='search/tweets', method='INVALID')

    @responses.activate
    def test_request_should_encode_boolean_as_lowercase_string(self):
        """Test that request() encodes a boolean parameter as a lowercase string"""
        endpoint = 'search/tweets'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url)

        self.api.request(endpoint, params={'include_entities': True})
        self.api.request(endpoint, params={'include_entities': False})

        self.assertEqual(url + '?include_entities=true', responses.calls[0].request.url)
        self.assertEqual(url + '?include_entities=false', responses.calls[1].request.url)

    @responses.activate
    def test_request_should_handle_string_or_number_parameter(self):
        """Test that request() encodes a numeric or string parameter correctly"""
        endpoint = 'search/tweets'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url)

        self.api.request(endpoint, params={'lang': 'es'})
        self.api.request(endpoint, params={'count': 50})

        self.assertEqual(url + '?lang=es', responses.calls[0].request.url)
        self.assertEqual(url + '?count=50', responses.calls[1].request.url)

    @responses.activate
    def test_request_should_encode_list_of_strings_as_string(self):
        """Test that request() encodes a list of strings as a comma-separated string"""
        endpoint = 'search/tweets'
        url = self.get_url(endpoint)
        location = ['37.781157', '-122.39872', '1mi']
        self.register_response(responses.GET, url)

        self.api.request(endpoint, params={'geocode': location})

        # requests url encodes the parameters so , is %2C
        self.assertEqual(url + '?geocode=37.781157%2C-122.39872%2C1mi', responses.calls[0].request.url)

    @responses.activate
    def test_request_should_encode_numeric_list_as_string(self):
        """Test that request() encodes a list of numbers as a comma-separated string"""
        endpoint = 'search/tweets'
        url = self.get_url(endpoint)
        location = [37.781157, -122.39872, '1mi']
        self.register_response(responses.GET, url)

        self.api.request(endpoint, params={'geocode': location})

        self.assertEqual(url + '?geocode=37.781157%2C-122.39872%2C1mi', responses.calls[0].request.url)

    @responses.activate
    def test_request_should_ignore_bad_parameter(self):
        """Test that request() ignores unexpected parameter types"""
        endpoint = 'search/tweets'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url)

        self.api.request(endpoint, params={'geocode': self})

        self.assertEqual(url, responses.calls[0].request.url)

    @responses.activate
    def test_request_should_handle_file_as_parameter(self):
        """Test that request() pulls a file out of params for requests lib"""
        endpoint = 'account/update_profile_image'
        url = self.get_url(endpoint)
        self.register_response(responses.POST, url)

        mock_file = StringIO("Twython test image")
        self.api.request(endpoint, method='POST', params={'image': mock_file})

        self.assertIn(b'filename="image"', responses.calls[0].request.body)
        self.assertIn(b"Twython test image", responses.calls[0].request.body)

    @responses.activate
    def test_request_should_put_params_in_body_when_post(self):
        """Test that request() passes params as data when the request is a POST"""
        endpoint = 'statuses/update'
        url = self.get_url(endpoint)
        self.register_response(responses.POST, url)

        self.api.request(endpoint, method='POST', params={'status': 'this is a test'})

        self.assertIn(b'status=this+is+a+test', responses.calls[0].request.body)
        self.assertNotIn('status=this+is+a+test', responses.calls[0].request.url)

    @responses.activate
    def test_get_uses_get_method(self):
        """Test Twython generic GET request works"""
        endpoint = 'account/verify_credentials'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url)

        self.api.get(endpoint)

        self.assertEqual(1, len(responses.calls))
        self.assertEqual(url, responses.calls[0].request.url)

    @responses.activate
    def test_post_uses_post_method(self):
        """Test Twython generic POST request works"""
        endpoint = 'statuses/update'
        url = self.get_url(endpoint)
        self.register_response(responses.POST, url)

        self.api.post(endpoint, params={'status': 'I love Twython!'})

        self.assertEqual(1, len(responses.calls))
        self.assertEqual(url, responses.calls[0].request.url)

    def test_raise_twython_error_on_request_exception(self):
        """Test if TwythonError is raised by a RequestException"""
        with mock.patch.object(requests.Session, 'get') as get_mock:
            # mocking an ssl cert error
            get_mock.side_effect = requests.RequestException("hostname 'example.com' doesn't match ...")
            self.assertRaises(TwythonError, self.api.get, 'https://example.com')

    @responses.activate
    def test_request_should_get_convert_json_to_data(self):
        """Test that Twython converts JSON data to a Python object"""
        endpoint = 'statuses/show'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url, body='{"id": 210462857140252672}')

        data = self.api.request(endpoint, params={'id': 210462857140252672})

        self.assertEqual({'id': 210462857140252672}, data)

    @responses.activate
    def test_request_should_raise_exception_with_invalid_json(self):
        """Test that Twython handles invalid JSON (though Twitter should not return it)"""
        endpoint = 'statuses/show'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url, body='{"id: 210462857140252672}')

        self.assertRaises(TwythonError, self.api.request, endpoint, params={'id': 210462857140252672})

    @responses.activate
    def test_request_should_handle_401(self):
        """Test that Twython raises an auth error on 401 error"""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url, body='{"errors":[{"message":"Error"}]}', status=401)

        self.assertRaises(TwythonAuthError, self.api.request, endpoint)

    @responses.activate
    def test_request_should_handle_400_for_missing_auth_data(self):
        """Test that Twython raises an auth error on 400 error when no oauth data sent"""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url,
                               body='{"errors":[{"message":"Bad Authentication data"}]}', status=400)

        self.assertRaises(TwythonAuthError, self.api.request, endpoint)

    @responses.activate
    def test_request_should_handle_400_that_is_not_auth_related(self):
        """Test that Twython raises a normal error on 400 error when unrelated to authorization"""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url,
                               body='{"errors":[{"message":"Bad request"}]}', status=400)

        self.assertRaises(TwythonError, self.api.request, endpoint)

    @responses.activate
    def test_request_should_handle_rate_limit(self):
        """Test that Twython raises an rate limit error on 429"""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url,
                               body='{"errors":[{"message":"Rate Limit"}]}', status=429)

        self.assertRaises(TwythonRateLimitError, self.api.request, endpoint)

    @responses.activate
    def test_get_lastfunction_header_should_return_header(self):
        """Test getting last specific header of the last API call works"""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url, adding_headers={'x-rate-limit-remaining': '37'})

        self.api.get(endpoint)

        value = self.api.get_lastfunction_header('x-rate-limit-remaining')
        self.assertEqual('37', value)
        value2 = self.api.get_lastfunction_header('does-not-exist')
        self.assertIsNone(value2)
        value3 = self.api.get_lastfunction_header('not-there-either', '96')
        self.assertEqual('96', value3)

    def test_get_lastfunction_header_should_raise_error_when_no_previous_call(self):
        """Test attempting to get a header when no API call was made raises a TwythonError"""
        self.assertRaises(TwythonError, self.api.get_lastfunction_header, 'no-api-call-was-made')

    @responses.activate
    def test_sends_correct_accept_encoding_header(self):
        """Test that Twython accepts compressed data."""
        endpoint = 'statuses/home_timeline'
        url = self.get_url(endpoint)
        self.register_response(responses.GET, url)

        self.api.get(endpoint)

        self.assertEqual(b'gzip, deflate', responses.calls[0].request.headers['Accept-Encoding'])

    # Static methods
    def test_construct_api_url(self):
        """Test constructing a Twitter API url works as we expect"""
        url = 'https://api.twitter.com/1.1/search/tweets.json'
        constructed_url = self.api.construct_api_url(url, q='#twitter')
        self.assertEqual(constructed_url, 'https://api.twitter.com/1.1/search/tweets.json?q=%23twitter')

    def test_encode(self):
        """Test encoding UTF-8 works"""
        self.api.encode('Twython is awesome!')

    def test_html_for_tweet(self):
        """Test HTML for Tweet returns what we want"""
        tweet_text = self.api.html_for_tweet(test_tweet_object)
        self.assertEqual(test_tweet_html, tweet_text)

    def test_html_for_tweet_expanded_url(self):
        """Test using expanded url in HTML for Tweet displays full urls"""
        tweet_text = self.api.html_for_tweet(test_tweet_object,
                                             use_expanded_url=True)
        # Make sure full url is in HTML
        self.assertTrue('http://google.com' in tweet_text)

    def test_html_for_tweet_short_url(self):
        """Test using expanded url in HTML for Tweet displays full urls"""
        tweet_text = self.api.html_for_tweet(test_tweet_object, False)
        # Make sure HTML doesn't contain the display OR expanded url
        self.assertTrue('http://google.com' not in tweet_text)
        self.assertTrue('google.com' not in tweet_text)

    def test_html_for_tweet_symbols(self):
        tweet_text = self.api.html_for_tweet(test_tweet_symbols_object)
        # Should only link symbols listed in entities:
        self.assertTrue('<a href="https://twitter.com/search?q=%24AAPL" class="twython-symbol">$AAPL</a>' in tweet_text)
        self.assertTrue('<a href="https://twitter.com/search?q=%24ANOTHER" class="twython-symbol">$ANOTHER</a>' not in tweet_text)

    def test_html_for_tweet_compatmode(self):
        tweet_text = self.api.html_for_tweet(test_tweet_compat_object)
        # link to compat web status link
        self.assertTrue(
            u'<a href="https://t.co/SRmsuks2ru" class="twython-url">twitter.com/i/web/status/7…</a>' in tweet_text)

    def test_html_for_tweet_extendedmode(self):
        tweet_text = self.api.html_for_tweet(test_tweet_extended_object)
        # full tweet rendered with suffix
        self.assertEqual(test_tweet_extended_html, tweet_text)



"""Entry point for the YellowAnt Developer CLI"""
from __future__ import print_function
import os
import sys
import click
import requests
import six

try: # python 2 imports
    import ConfigParser
except: # python 3 imports
    import configparser as ConfigParser

CONTEXT_SETTINGS = dict(auto_envvar_prefix='yacli')


class Context(object):
    """YellowAnt Dev CLI common methods"""
    YELLOWANT_HOST = "https://yellowant.com"

    def __init__(self):
        self.verbose = False
        self.requests = None
        self.config_file = None
        self.token = None
        self.host = None

        try:
            self.get_auth_from_config_file()
        except:
            self.token = None
            self.host = None
    
    def is_auth_valid(self):
        """Check if authentication credentials are present"""
        if self.token is None or self.host is None:
            raise click.ClickException("You cannot perform this command without authentication.\n" +\
                                       "Run 'yellowant auth' to remember authentication credentials, or provide '--token' and '--host' flag values before this command")
    
    def get_auth_from_config_file(self):
        """Load authentication credentials from config file"""
        # get home directory for UNIX or windows systems
        self.config_file = os.path.join((os.getenv("HOME") or ("C:" + os.getenv("HOMEPATH"))), ".yellowant")
        config = ConfigParser.ConfigParser()
        config.read(self.config_file)
        self.token = config.get("YELLOWANT_DEV", "token")
        self.host = config.get("YELLOWANT_DEV", "host")
    
    def create_requests_object(self):
        """Add headers to requests object"""
        self.requests = requests.Session()
        self.requests.headers.update({
            "Authorization": "Token {}".format(self.token),
            "Content-type": "application/json"
        })
    
    def build_endpoint(self, endpoint):
        """Join the host, api, and enpoint"""
        if self.host[-1] == "/":
            # remove trailing slash
            self.host = self.host[:-1]
        return "".join([self.host, endpoint])
    
    def get(self, endpoint):
        """requests GET method wrapper"""
        self.create_requests_object()
        return self.requests.get(self.build_endpoint(endpoint))

    def post(self, endpoint, data=None):
        """requests POST method wrapper"""
        self.create_requests_object()
        return self.requests.post(self.build_endpoint(endpoint), data=data)

    def put(self, endpoint, data=None):
        """requests PUT method wrapper"""
        self.create_requests_object()
        return self.requests.put(self.build_endpoint(endpoint), data=data)

    def log(self, msg, *args):
        """Logs a message to stderr."""
        if args:
            msg %= args
        click.echo(msg, file=sys.stderr)

    def vlog(self, msg, *args):
        """Logs a message to stderr only if verbose is enabled."""
        if self.verbose:
            self.log(msg, *args)


pass_context = click.make_pass_decorator(Context, ensure=True)
plugin_folder = os.path.abspath(os.path.join(os.path.dirname(__file__),'commands'))


class YellowAntDevCLI(click.MultiCommand):
    """Base Command"""
    def list_commands(self, ctx):
        rv = []
        for filename in os.listdir(plugin_folder):
            if filename.endswith('.py') and "__init__" not in filename:
                rv.append(filename[:-3])
        rv.sort()
        return rv

    def get_command(self, ctx, name):
        ns = {}
        fn = os.path.join(plugin_folder, name + '.py')
        with open(fn) as f:
            code = compile(f.read(), fn, 'exec')
            eval(code, ns, ns)
        return ns['cli']

    # def list_commands(self, ctx):
    #     rv = []
    #     for filename in os.listdir(plugin_folder):
    #         if filename.endswith('.py') and filename.startswith('cmd_'):
    #             rv.append(filename[4:-3])
    #     rv.sort()
    #     print(rv)
    #     return rv

    # def get_command(self, ctx, name):
    #     try:
    #         if sys.version_info[0] == 2:
    #             name = name.encode('ascii', 'replace')
    #         mod = __import__('yacli.commands.cmd_' + name,
    #                          None, None, ['cli'])
    #     except ImportError:
    #         return
    #     return mod.cli


@click.command(cls=YellowAntDevCLI, context_settings=CONTEXT_SETTINGS)
@click.option("--token", help="YellowAnt Developer Token")
@click.option("--host", help="Yellowant Host URL")
@click.option('-v', '--verbose', is_flag=True,
              help='Enables verbose mode.')
@pass_context
def cli(ctx, token=None, host=None, verbose=False):
    """This tool allows YellowAnt application developers to manage their applications."""
    ctx.verbose = verbose
    if token is not None:
        ctx.token = token
    if host is not None:
        ctx.host = host


# -*- coding: utf-8 -*-

"""
YellowAnt.api
~~~~~~~~~~~

This module contains functionality for access to core YellowAnt API calls,
YellowAnt Authentication, and miscellaneous methods that are useful when
dealing with the YellowAnt API
"""

import warnings
import re
import os

import requests
from requests.auth import HTTPBasicAuth
from requests_oauthlib import OAuth1, OAuth2

from . import __version__
from .advisory import YellowAntDeprecationWarning
from .compat import json, urlencode, parse_qsl, quote_plus, str, is_py2
from .endpoints import Endpoints
from .exceptions import YellowAntError, YellowAntAuthError, YellowAntRateLimitError
from .helpers import _transparent_params

warnings.simplefilter('always', YellowAntDeprecationWarning)  # For Python 2.7 >


class YellowAnt(Endpoints, object):
    def __init__(self, app_key=None, app_secret=None, access_token=None, redirect_uri = None,
                 token_type='bearer', oauth_version=2, api_version='1.0',
                 client_args=None, auth_endpoint='authenticate', api_url=None):
        """Instantiates an instance of YellowAnt. Takes optional parameters for
        authentication and such (see below).

        :param app_key: (optional) Your applications key
        :param app_secret: (optional) Your applications secret key
        :param access_token: (optional) When using **OAuth 2**, provide a
        valid access token if you have one

        """

        # API urls, OAuth urls and API version; needed for hitting that there
        # API.
        self.api_version = api_version
        self.api_url = os.environ.get('YELLOWANT_API_URL') or api_url
        if self.api_url is None:
            self.api_url = 'https://api.yellowant.com/api/%s'
        self.app_key = app_key
        self.app_secret = app_secret
        self.redirect_uri = redirect_uri
        self.access_token = access_token

        # OAuth 1
        self.request_token_url = self.api_url % 'oauth/request_token'
        self.access_token_url = self.api_url % 'oauth/access_token'
        self.authenticate_url = self.api_url % ('oauth/%s' % auth_endpoint)

        self.oauth_version = 2

        self.request_token_url = self.api_url % 'oauth2/token/'

        self.client_args = client_args or {}
        default_headers = {'content-type':'application/json', 'User-Agent': 'YellowAnt-Python' + __version__}
        if 'headers' not in self.client_args:
            # If they didn't set any headers, set our defaults for them
            self.client_args['headers'] = default_headers
        elif 'User-Agent' not in self.client_args['headers']:
            # If they set headers, but didn't include User-Agent
            self.client_args['headers'].update(default_headers)

        auth = None
        token = {'token_type': token_type,
                 'access_token': self.access_token}
        auth = OAuth2(self.app_key, token=token)

        self.client = requests.Session()
        self.client.auth = auth

        # Make a copy of the client args and iterate over them
        # Pop out all the acceptable args at this point because they will
        # Never be used again.
        client_args_copy = self.client_args.copy()
        for k, v in client_args_copy.items():
            if k in ('cert', 'hooks', 'max_redirects', 'proxies'):
                setattr(self.client, k, v)
                self.client_args.pop(k)  # Pop, pop!

        # Headers are always present, so we unconditionally pop them and merge
        # them into the session headers.
        self.client.headers.update(self.client_args.pop('headers'))

        self._last_call = None

    def __repr__(self):
        return '<YellowAnt: %s>' % (self.app_key)

    def _request(self, url, method='GET', params=None, api_call=None):
        """Internal request method"""
        method = method.lower()
        params = params or {}

        func = getattr(self.client, method)
        params, files = _transparent_params(params)


        requests_args = {}
        for k, v in self.client_args.items():
            # Maybe this should be set as a class variable and only done once?
            if k in ('timeout', 'allow_redirects', 'stream', 'verify'):
                requests_args[k] = v

        if method == 'get':
            requests_args['params'] = params
        else:
            requests_args.update({
                'data': json.dumps(params),
                'files': files,
            })
        try:
            response = func(url, **requests_args)
        except requests.RequestException as e:
            raise YellowAntError(str(e))

        # create stash for last function intel
        self._last_call = {
            'api_call': api_call,
            'api_error': None,
            'cookies': response.cookies,
            'headers': response.headers,
            'status_code': response.status_code,
            'url': response.url,
            'content': response.text,
        }

        # greater than 304 (not modified) is an error
        if response.status_code > 304:
            error_message = self._get_error_message(response)
            self._last_call['api_error'] = error_message

            ExceptionType = YellowAntError
            if response.status_code == 429:
                ExceptionType = YellowAntRateLimitError
            elif response.status_code == 401 or 'Bad Authentication data' \
                    in error_message:
                ExceptionType = YellowAntAuthError

            raise ExceptionType(
                error_message,
                error_code=response.status_code,
                retry_after=response.headers.get('X-Rate-Limit-Reset'))

        try:
            if response.status_code == 204:
                content = response.content
            else:
                content = response.json()
        except ValueError:
            raise YellowAntError('Response was not valid JSON. \
                               Unable to decode.')

        return content

    def _get_error_message(self, response):
        """Parse and return the first error message"""

        error_message = 'An error occurred processing your request.'
        try:
            content = response.json()
            # {"errors":[{"code":34,"message":"Sorry,
            # that page does not exist"}]}
            error_message = content['errors'][0]['message']
        except TypeError:
            error_message = content['errors']
        except ValueError:
            # bad json data from Twitter for an error
            pass
        except (KeyError, IndexError):
            # missing data so fallback to default message
            pass

        return error_message

    def request(self, endpoint, method='GET', params=None, version='1.1'):
        if endpoint.startswith('http://'):
            raise YellowAntError('api.yellowant.com is restricted to SSL/TLS traffic.')

        if endpoint.startswith('https://'):
            url = endpoint
        else:
            url = self.api_url % endpoint

        content = self._request(url, method=method, params=params,
                                api_call=url)

        return content

    def get(self, endpoint, params=None, version='1'):
        """Shortcut for GET requests via :class:`request`"""
        return self.request(endpoint, params=params, version=version)

    def post(self, endpoint, params=None, version='1'):
        """Shortcut for POST requests via :class:`request`"""
        return self.request(endpoint, 'POST', params=params, version=version)

    def delete(self, endpoint, params=None, version='1'):
        """Shortcut for POST requests via :class:`request`"""
        return self.request(endpoint, 'DELETE', params=params, version=version)

    def put(self, endpoint, params=None, version='1'):
        """Shortcut for POST requests via :class:`request`"""
        return self.request(endpoint, 'PUT', params=params, version=version)

    def patch(self, endpoint, params=None, version='1'):
        """Shortcut for POST requests via :class:`request`"""
        return self.request(endpoint, 'PATCH', params=params, version=version)

    def get_lastfunction_header(self, header, default_return_value=None):
        """Returns a specific header from the last API call
        This will return None if the header is not present

        :param header: (required) The name of the header you want to get
                       the value of

        Most useful for the following header information:
            x-rate-limit-limit,
            x-rate-limit-remaining,
            x-rate-limit-class,
            x-rate-limit-reset

        """
        if self._last_call is None:
            raise YellowAntError('This function must be called after an API call. \
                               It delivers header information.')

        return self._last_call['headers'].get(header, default_return_value)

    def get_authentication_tokens(self, callback_url=None):

        if self.oauth_version != 1:
            raise YellowAntError('This method can only be called when your \
                               OAuth version is 1.0.')

        request_args = {}
        if callback_url:
            request_args['oauth_callback'] = callback_url
        response = self.client.get(self.request_token_url, params=request_args)

        if response.status_code == 401:
            raise YellowAntAuthError(response.content,
                                     error_code=response.status_code)
        elif response.status_code != 200:
            raise YellowAntError(response.content,
                                 error_code=response.status_code)

        request_tokens = dict(parse_qsl(response.content.decode('utf-8')))
        if not request_tokens:
            raise YellowAntError('Unable to decode request tokens.')

        oauth_callback_confirmed = request_tokens.get('oauth_callback_confirmed') \
                                   == 'true'

        auth_url_params = {
            'oauth_token': request_tokens['oauth_token'],
        }

        # Use old-style callback argument if server didn't accept new-style
        if callback_url and not oauth_callback_confirmed:
            auth_url_params['oauth_callback'] = self.callback_url

        request_tokens['auth_url'] = self.authenticate_url + \
                                     '?' + urlencode(auth_url_params)

        return request_tokens

    def get_authorized_tokens(self, oauth_verifier):
        if self.oauth_version != 1:
            raise YellowAntError('This method can only be called when your \
                               OAuth version is 1.0.')

        response = self.client.get(self.access_token_url,
                                   params={'oauth_verifier': oauth_verifier},
                                   headers={'Content-Type': 'application/\
                                   json'})

        if response.status_code == 401:
            try:
                try:
                    # try to get json
                    content = response.json()
                except AttributeError:  # pragma: no cover
                    # if unicode detected
                    content = json.loads(response.content)
            except ValueError:
                content = {}

            raise YellowAntError(content.get('error', 'Invalid / expired To \
            ken'), error_code=response.status_code)

        authorized_tokens = dict(parse_qsl(response.content.decode('utf-8')))
        if not authorized_tokens:
            raise YellowAntError('Unable to decode authorized tokens.')

        return authorized_tokens  # pragma: no cover

    def get_access_token(self, code):

        if self.oauth_version != 2:
            raise YellowAntError('This method can only be called when your \
                               OAuth version is 2.0.')

        data = {'grant_type': 'authorization_code', 'client_id': self.app_key, 'client_secret': self.app_secret,
                'code': code, 'redirect_uri': self.redirect_uri}
        try:
            client = requests.Session()
            response = client.post(self.request_token_url,
                                        data=data)
            content = response.content.decode('utf-8')
            try:
                content = content.json()
            except AttributeError:
                content = json.loads(content)
                #access_token = content['access_token']
        except Exception as e:
            raise YellowAntError(str(e))
        else:
            return content


    @staticmethod
    def construct_api_url(api_url, **params):
        querystring = []
        params, _ = _transparent_params(params or {})
        params = requests.utils.to_key_val_list(params)
        for (k, v) in params:
            querystring.append(
                '%s=%s' % (YellowAnt.encode(k), quote_plus(YellowAnt.encode(v)))
            )
        return '%s?%s' % (api_url, '&'.join(querystring))


    @staticmethod
    def unicode2utf8(text):
        try:
            if is_py2 and isinstance(text, str):
                text = text.encode('utf-8')
        except:
            pass
        return text

    @staticmethod
    def encode(text):
        if is_py2 and isinstance(text, (str)):
            return YellowAnt.unicode2utf8(text)
        return str(text)

